//-----------------------------------------------------------------------------
//
// File:    QCDEncodeDLL.cpp
//
// About:   See QCDOutputDLL.h
//
// Authors: Written by Paul Quinn
//
//  QCD multimedia player application Software Development Kit Release 1.0.
//
//  Copyright (C) 1997-2002 Quinnware
//
//  This code is free.  If you redistribute it in any form, leave this notice 
//  here.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
//-----------------------------------------------------------------------------

#include "QCDEncodeDLL.h"

HINSTANCE       hInstance, hBrandInstance;
HWND            hwndParent;
QCDModInitEnc   QCDCallbacks;

//int srate, nch, bps;
bool initialized;
unsigned long start_time;
double decoded_seconds;

#define NULL_VERSION "v1.0"

//-----------------------------------------------------------------------------

BOOL WINAPI DllMain(HINSTANCE hInst, DWORD fdwReason, LPVOID pRes)
{
    if (fdwReason == DLL_PROCESS_ATTACH)
    {
        hInstance = hInst;
    }
    return TRUE;
}

//-----------------------------------------------------------------------------

PLUGIN_API QCDModInitEnc* ENCODEDLL_ENTRY_POINT()
{
    QCDCallbacks.version = PLUGIN_API_VERSION;

    QCDCallbacks.toModule.Initialize        = Initialize;
    QCDCallbacks.toModule.ShutDown          = ShutDown;
    QCDCallbacks.toModule.Open              = Open;
    QCDCallbacks.toModule.Write             = Write;
    QCDCallbacks.toModule.Drain             = Drain;
    QCDCallbacks.toModule.Complete          = Complete;
    QCDCallbacks.toModule.Configure         = Configure;
    QCDCallbacks.toModule.About             = About;

    return &QCDCallbacks;
}

//-----------------------------------------------------------------------------

static int print_speed()
{
    unsigned long cur_time = GetTickCount();
    double total_time = (double)(cur_time - start_time) / 1000.0;

    if ( cur_time < start_time ) total_time = (double)(cur_time + (((__int64)1<<32) - start_time)) / 1000.0;

    if ( decoded_seconds == 0.0 || total_time == 0.0 ) return 0;

    double speed = decoded_seconds / total_time;
    int x = (int)(speed * 100.);

    char temp[1024];
    wsprintfA ( temp, "decoding took %u milliseconds, speed %i.%02ix", (unsigned int)(total_time * 1000. + 0.5), x/100, x%100 );

    MessageBoxA ( 0, temp, "Speed Meter", MB_OK );

    return 1;
}

//-----------------------------------------------------------------------------

BOOL Initialize(QCDModInfo *modInfo, int flags)
{
    modInfo->moduleString = "Speed Meter " NULL_VERSION;
    modInfo->moduleExtensions = "Speed Meter";

    hwndParent = (HWND)QCDCallbacks.Service(opGetParentWnd, 0, 0, 0);

    char inifile[MAX_PATH];
    QCDCallbacks.Service(opGetPluginSettingsFile, inifile, MAX_PATH, 0);

    //
    // TODO: all your plugin initialization here
    //

    
    // return TRUE for successful initialization
    return TRUE;
}

//-----------------------------------------------------------------------------

void ShutDown(int flags)
{
    //
    // TODO : Cleanup
    // make plugin ready to be unloaded
    //
}

//-----------------------------------------------------------------------------

BOOL Open(LPCSTR outFile, LPCSTR srcFile, WAVEFORMATEX *wf, LPSTR openedFilename, int openedFilenameSize)
{
    //
    // TODO : 
    // - verify format of audio data that will be received to make sure it's compatable
    //      return FALSE if not;
    // - append extension to outFile 
    // - verify and open outFile to write data to
    // - srcFile is available to notify what will be the source of incoming data
    // 
    // - copy filename of opened file back to openedFilename (so player knows what this plugin named the file)
    //
    // Return TRUE for success, FALSE for failure


    // The following is recommended since encoders take a lot of CPU and 
    // this thread is generally set to a high priority by the input plugin.
    // Lowering the priority back to normal will make the player run smoother.

    initialized = false;
    //srate = samplerate;
    //nch = numchannels;
    //bps = bitspersamp;

    SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_NORMAL);

    return TRUE;
}

//-----------------------------------------------------------------------------

BOOL Write(WriteDataStruct* writeData)
{
    //
    // TODO : Use the raw wave audio data passed from the player.
    //
    // Return value - one of the follwing:
    //
    // TRUE     - write succeeded
    // FALSE    - write failed
    //
    // Note: this call can block (eg: to wait for available space to write)

    if ( !initialized ) {
        decoded_seconds = 0;
        initialized = true;
        start_time = GetTickCount();
    }

    decoded_seconds += (double)writeData->numsamples / (double)writeData->srate;

    return TRUE;
}

//-----------------------------------------------------------------------------

BOOL Drain(int flags)
{
    //
    // TODO : Write any remaining data to output.
    //
    // Return TRUE for success, FALSE for failure

    return TRUE;
}

//-----------------------------------------------------------------------------

BOOL Complete(int flags)
{
    //
    // TODO : Finish and close the output.
    //
    // Return TRUE for success, FALSE for failure

    if ( initialized ) print_speed();

    return TRUE;
}

//-----------------------------------------------------------------------------

void Configure(int flags)
{
    //
    // TODO : Show "configure" dialog.
    //
}

//-----------------------------------------------------------------------------

void About(int flags)
{
    //
    // TODO : Show "about" dialog.
    //

    MessageBoxA ( hwndParent, "Speed Meter - QCD decode speed tester " NULL_VERSION "\n"
                              "Copyright (c) 2003 Case\n",
                              "About", MB_OK );
}

//-----------------------------------------------------------------------------

